@@ -7,7 +7,8 @@ from consts import *
 from ModuleHandler import *
 from ModuleBalancer import NOTE_BALANCER
 
-NOTE_REUSE_MAX = "Maximum number of connection per server that the proxy can try to keep opened."
+NOTE_REUSE_MAX       = "Maximum number of connection per server that the proxy can try to keep opened."
+NOTE_ALLOW_KEEPALIVE = "Allow the server to use Keep-alive connections with the back-end servers."
 
 HELPS = [
     ('modules_handlers_proxy', "Reverse Proxy")
@@ -15,7 +16,7 @@ HELPS = [
 
 class ModuleProxy (ModuleHandler):
     PROPERTIES = [
-        'balancer',
+        'balancer', 'in_allow_keepalive',
         'in_header_add',      'out_header_add',
         'in_header_hide',     'out_header_hide',
         'in_rewrite_request', 'out_rewrite_request'
@@ -50,6 +51,7 @@ class ModuleProxy (ModuleHandler):
     def _render_general (self):
         table = TableProps()
         self.AddPropEntry (table, 'Reuse connections', '%s!reuse_max'%(self._prefix), NOTE_REUSE_MAX)
+        self.AddPropCheck (table, 'Allow Keepalive',   '%s!in_allow_keepalive'%(self._prefix), True, NOTE_ALLOW_KEEPALIVE)
         return str(table)
 
     def _render_request (self):
@@ -201,4 +203,4 @@ class ModuleProxy (ModuleHandler):
             props._op_apply_changes (uri, post)
 
         # And CGI changes
-        self.ApplyChangesPrefix (self._prefix, [], post)
+        self.ApplyChangesPrefix (self._prefix, ['in_allow_keepalive'], post)
