@@ -66,7 +66,9 @@ static const char *month[] = {
 
 
 ret_t
-cherokee_logger_ncsa_new (cherokee_logger_t **logger, cherokee_config_node_t *config)
+cherokee_logger_ncsa_new (cherokee_logger_t         **logger,
+			  cherokee_virtual_server_t  *vsrv,
+			  cherokee_config_node_t     *config)
 {
 	ret_t ret;
 	CHEROKEE_NEW_STRUCT (n, logger_ncsa);
@@ -85,7 +87,10 @@ cherokee_logger_ncsa_new (cherokee_logger_t **logger, cherokee_config_node_t *co
 	LOGGER(n)->write_string   = (logger_func_write_string_t) cherokee_logger_ncsa_write_string;
 	LOGGER(n)->write_error_fd = (logger_func_write_error_fd_t)  cherokee_logger_ncsa_write_error_fd;
 
-	ret = cherokee_logger_ncsa_init_base (n, config);
+	n->writer_access = NULL;
+	n->writer_error  = NULL;
+
+	ret = cherokee_logger_ncsa_init_base (n, VSERVER(vsrv), config);
 	if (unlikely(ret < ret_ok)) return ret;
 
 	/* Return the object
@@ -96,15 +101,14 @@ cherokee_logger_ncsa_new (cherokee_logger_t **logger, cherokee_config_node_t *co
 
 
 ret_t 
-cherokee_logger_ncsa_init_base (cherokee_logger_ncsa_t *logger, cherokee_config_node_t *config)
+cherokee_logger_ncsa_init_base (cherokee_logger_ncsa_t    *logger,
+				cherokee_virtual_server_t *vsrv,
+				cherokee_config_node_t    *config)
 {
 	ret_t                   ret;
 	long                   *this_timezone = NULL;
 	cherokee_config_node_t *subconf;
 
-	/* Init the local vars
-	 */
-
 	/* Get the timezone reference and compute tz offset
 	 */
 	this_timezone = cherokee_get_timezone_ref();
@@ -122,24 +126,20 @@ cherokee_logger_ncsa_init_base (cherokee_logger_ncsa_t *logger, cherokee_config_
 
 	/* Init the logger writers
 	 */
-	ret = cherokee_logger_writer_init (&logger->writer_access);
-	if (ret != ret_ok) return ret;
-
-	ret = cherokee_logger_writer_init (&logger->writer_error);
-	if (ret != ret_ok) return ret;
-
-	/* Configure them
-	 */
 	ret = cherokee_config_node_get (config, "access", &subconf);
 	if (ret == ret_ok) {
-		ret = cherokee_logger_writer_configure (&logger->writer_access, subconf);
-		if (ret != ret_ok) return ret;
+		ret = cherokee_server_get_log_writer (VSERVER_SRV(vsrv), subconf, &logger->writer_access);
+		if (ret != ret_ok) {
+			return ret_error;
+		}
 	}
 
 	ret = cherokee_config_node_get (config, "error", &subconf);
 	if (ret == ret_ok) {
-		ret = cherokee_logger_writer_configure (&logger->writer_error, subconf);
-		if (ret != ret_ok) return ret;
+		ret = cherokee_server_get_log_writer (VSERVER_SRV(vsrv), subconf, &logger->writer_error);
+		if (ret != ret_ok) {
+			return ret_error;
+		}
 	}
 
 	return ret_ok;
@@ -151,10 +151,10 @@ cherokee_logger_ncsa_init (cherokee_logger_ncsa_t *logger)
 {
 	ret_t ret;
 
-	ret = cherokee_logger_writer_open (&logger->writer_access);
+	ret = cherokee_logger_writer_open (logger->writer_access);
 	if (ret != ret_ok) return ret;
 
-	ret = cherokee_logger_writer_open (&logger->writer_error);
+	ret = cherokee_logger_writer_open (logger->writer_error);
 	if (ret != ret_ok) return ret;
 
 	return ret_ok;
@@ -164,18 +164,10 @@ cherokee_logger_ncsa_init (cherokee_logger_ncsa_t *logger)
 ret_t
 cherokee_logger_ncsa_free (cherokee_logger_ncsa_t *logger)
 {
-	ret_t ret;
-
 	cherokee_buffer_mrproper (&logger->now_dtm);
 	cherokee_buffer_mrproper (&logger->referer);
 	cherokee_buffer_mrproper (&logger->useragent);
 
-	ret = cherokee_logger_writer_mrproper (&logger->writer_access);
-	if (ret != ret_ok) return ret;
-
-	ret = cherokee_logger_writer_mrproper (&logger->writer_error);
-	if (ret != ret_ok) return ret;
-
 	return ret_ok;
 }
 
@@ -183,7 +175,7 @@ cherokee_logger_ncsa_free (cherokee_logger_ncsa_t *logger)
 ret_t 
 cherokee_logger_ncsa_flush (cherokee_logger_ncsa_t *logger)
 {
-	return cherokee_logger_writer_flush (&logger->writer_access);
+	return cherokee_logger_writer_flush (logger->writer_access, false);
 }
 
 
@@ -218,14 +210,10 @@ build_log_string (cherokee_logger_ncsa_t *logger, cherokee_connection_t *cnt, ch
 				pnow_tm->tm_min, 
 				pnow_tm->tm_sec,
 				(logger->tz < 0) ? '-' : '+', 
-				(int) (logger->tz / 60), 
-				(int) (logger->tz % 60)
-				);
+				(int) (abs(logger->tz) / 60), 
+				(int) (abs(logger->tz) % 60));
 	}
 
-	memset (ipaddr, 0, sizeof(ipaddr));
-	cherokee_socket_ntop (&cnt->socket, ipaddr, sizeof(ipaddr)-1);
-
 	/* Look for the user
 	 */
 	if (cnt->validator && !cherokee_buffer_is_empty (&cnt->validator->user)) {
@@ -253,15 +241,27 @@ build_log_string (cherokee_logger_ncsa_t *logger, cherokee_connection_t *cnt, ch
 	 * "%s - %s [%02d/%s/%d:%02d:%02d:%02d %c%02d%02d] \"%s %s %s\" %d "
 	 * FMT_OFFSET
 	 */
-	cherokee_buffer_add        (buf, ipaddr, strlen(ipaddr));
-	cherokee_buffer_add_str    (buf, " - ");
-	cherokee_buffer_add        (buf, username, username_len);
+	if (cherokee_buffer_is_empty (&cnt->logger_real_ip)) {
+		memset (ipaddr, 0, sizeof(ipaddr));
+		cherokee_socket_ntop (&cnt->socket, ipaddr, sizeof(ipaddr)-1);
+		cherokee_buffer_add (buf, ipaddr, strlen(ipaddr));
+	} else {
+		cherokee_buffer_add_buffer (buf, &cnt->logger_real_ip);
+	}
+
+	cherokee_buffer_add_str (buf, " - ");
+	cherokee_buffer_add     (buf, username, username_len);
+
 	/* " [date time] "
 	 */
 	cherokee_buffer_add_buffer (buf, &logger->now_dtm);
 	cherokee_buffer_add        (buf, method, method_len);
 	cherokee_buffer_add_char   (buf, ' ');
 	cherokee_buffer_add_buffer (buf, request);
+	if (! cherokee_buffer_is_empty (&cnt->query_string)) {
+		cherokee_buffer_add_char   (buf, '?');
+		cherokee_buffer_add_buffer (buf, &cnt->query_string);
+	}
 	cherokee_buffer_add_char   (buf, ' ');
 	cherokee_buffer_add        (buf, version, version_len);
 	cherokee_buffer_add_str    (buf, "\" ");
@@ -313,22 +313,30 @@ cherokee_logger_ncsa_write_string (cherokee_logger_ncsa_t *logger, const char *s
 	ret_t              ret;
 	cherokee_buffer_t *log;
 
-	ret = cherokee_logger_writer_get_buf (&logger->writer_access, &log);
-	if (unlikely (ret != ret_ok)) return ret;
+	/* Get the buffer
+	 */
+	cherokee_logger_writer_get_buf (logger->writer_access, &log);
 
 	ret = cherokee_buffer_add (log, string, strlen(string));
- 	if (unlikely (ret != ret_ok)) return ret;
+ 	if (unlikely (ret != ret_ok))
+		goto error;
 
 	/* Flush buffer if full
 	 */  
-  	if (log->len < logger->writer_access.max_bufsize)
-		return ret_ok;
+  	if (log->len < logger->writer_access->max_bufsize)
+		goto ok;
 
-	ret = cherokee_logger_writer_flush (&logger->writer_access);
+	ret = cherokee_logger_writer_flush (logger->writer_access, true);
 	if (unlikely (ret != ret_ok))
-		return ret;
+		goto error;
 
+ok:
+	cherokee_logger_writer_release_buf (logger->writer_access);
 	return ret_ok;
+
+error:
+	cherokee_logger_writer_release_buf (logger->writer_access);
+	return ret_error;
 }
 
 
@@ -340,24 +348,30 @@ cherokee_logger_ncsa_write_access (cherokee_logger_ncsa_t *logger, cherokee_conn
 	
 	/* Get the buffer
 	 */
-	ret = cherokee_logger_writer_get_buf (&logger->writer_access, &log);
-	if (unlikely (ret != ret_ok)) return ret;
+	cherokee_logger_writer_get_buf (logger->writer_access, &log);
 
 	/* Add the new string
 	 */
 	ret = build_log_string (logger, cnt, log);
-	if (unlikely (ret != ret_ok)) return ret;
+	if (unlikely (ret != ret_ok)) 
+		goto error;
 
 	/* Flush buffer if full
 	 */  
-	if (log->len < logger->writer_access.max_bufsize)
-		return ret_ok;
+	if (log->len < logger->writer_access->max_bufsize)
+		goto ok;
 
-	ret = cherokee_logger_writer_flush (&logger->writer_access);
+	ret = cherokee_logger_writer_flush (logger->writer_access, true);
 	if (unlikely (ret != ret_ok))
-		return ret;
+		goto error;
 
+ok:
+	cherokee_logger_writer_release_buf (logger->writer_access);
 	return ret_ok;
+
+error:
+	cherokee_logger_writer_release_buf (logger->writer_access);
+	return ret_error;
 }
 
 
@@ -369,31 +383,36 @@ cherokee_logger_ncsa_write_error (cherokee_logger_ncsa_t *logger, cherokee_conne
 
 	/* Get the buffer
 	 */
-	ret = cherokee_logger_writer_get_buf (&logger->writer_error, &log);
-	if (unlikely (ret != ret_ok)) return ret;
+	cherokee_logger_writer_get_buf (logger->writer_error, &log);
 
 	/* Add the new string
 	 */
 	ret = build_log_string (logger, cnt, log);
-	if (unlikely (ret != ret_ok)) return ret;
+	if (unlikely (ret != ret_ok))
+		goto error;
 
 	/* It's an error. Flush it!
 	 */
-	ret = cherokee_logger_writer_flush (&logger->writer_error);
+	ret = cherokee_logger_writer_flush (logger->writer_error, true);
 	if (unlikely (ret != ret_ok))
-		return ret;
+		goto error;
 
+	cherokee_logger_writer_release_buf (logger->writer_error);
 	return ret_ok;
+
+error:
+	cherokee_logger_writer_release_buf (logger->writer_error);
+	return ret_error;
 }
 
 
 ret_t 
 cherokee_logger_ncsa_write_error_fd (cherokee_logger_ncsa_t *logger, int fd)
 {
-	if ((logger->writer_error.fd != -1) &&
-	    (logger->writer_error.fd != fd))
+	if ((logger->writer_error->fd != -1) &&
+	    (logger->writer_error->fd != fd))
 	{
-		dup2 (logger->writer_error.fd, fd);
+		dup2 (logger->writer_error->fd, fd);
 	}
 
 	return ret_ok;
@@ -406,8 +425,8 @@ cherokee_logger_ncsa_reopen (cherokee_logger_ncsa_t *logger)
 	ret_t ret1;
 	ret_t ret2;
 
-	ret1 = cherokee_logger_writer_reopen (&logger->writer_access);
-	ret2 = cherokee_logger_writer_reopen (&logger->writer_error);
+	ret1 = cherokee_logger_writer_reopen (logger->writer_access);
+	ret2 = cherokee_logger_writer_reopen (logger->writer_error);
 
 	if (ret1 != ret_ok)
 		return ret1;
