@@ -36,18 +36,19 @@
 #include "connection.h"
 #include "logger.h"
 #include "logger_writer.h"
+#include "virtual_server.h"
 
 typedef struct {
-	cherokee_logger_t        logger;	
-	cherokee_boolean_t       header_added;
-	time_t                   now_time;
-	cherokee_buffer_t        now_buf;
-	cherokee_logger_writer_t writer;
+	cherokee_logger_t         logger;	
+	cherokee_boolean_t        header_added;
+	time_t                    now_time;
+	cherokee_buffer_t         now_buf;
+	cherokee_logger_writer_t *writer;
 } cherokee_logger_w3c_t;
 
 
-ret_t cherokee_logger_w3c_new (cherokee_logger_t **logger, cherokee_config_node_t *config);
-ret_t cherokee_logger_w3c_init_base (cherokee_logger_w3c_t *logger, cherokee_config_node_t *config);
+ret_t cherokee_logger_w3c_new (cherokee_logger_t **logger, cherokee_virtual_server_t *vsrv, cherokee_config_node_t *config);
+ret_t cherokee_logger_w3c_init_base (cherokee_logger_w3c_t *logger, cherokee_virtual_server_t *vsrv, cherokee_config_node_t *config);
 
 /* virtual methods implementation
  */
