@@ -51,7 +51,6 @@ cherokee_virtual_server_new (cherokee_virtual_server_t **vserver, void *server)
 	n->default_handler = NULL;
 	n->error_handler   = NULL;
 	n->logger          = NULL;
-	n->logger_props    = NULL;
 	n->priority        = 0;
 	n->keepalive       = true;
 	n->cryptor         = NULL;
@@ -82,6 +81,7 @@ cherokee_virtual_server_new (cherokee_virtual_server_t **vserver, void *server)
 	cherokee_buffer_init (&n->server_key);
 	cherokee_buffer_init (&n->certs_ca);
 	cherokee_buffer_init (&n->req_client_certs);
+	cherokee_buffer_init (&n->ciphers);
 
 	ret = cherokee_buffer_init (&n->root);
 	if (unlikely(ret < ret_ok))
@@ -111,6 +111,7 @@ cherokee_virtual_server_free (cherokee_virtual_server_t *vserver)
 	cherokee_buffer_mrproper (&vserver->server_key);
 	cherokee_buffer_mrproper (&vserver->certs_ca);
 	cherokee_buffer_mrproper (&vserver->req_client_certs);
+	cherokee_buffer_mrproper (&vserver->ciphers);
 
 	if (vserver->error_handler != NULL) {
 		cherokee_config_entry_free (vserver->error_handler);
@@ -131,16 +132,6 @@ cherokee_virtual_server_free (cherokee_virtual_server_t *vserver)
 	cherokee_vserver_names_mrproper (&vserver->domains);
 
 	cherokee_buffer_mrproper (&vserver->root);
-
-	if (vserver->logger != NULL) {
-		cherokee_logger_free (vserver->logger);
-		vserver->logger = NULL;
-	}
-	if (vserver->logger_props != NULL) {
-		cherokee_avl_free (vserver->logger_props, NULL); /* FIXIT */
-		vserver->logger_props = NULL;
-	}
-
 	cherokee_buffer_mrproper (&vserver->userdir);
 
 	/* Destroy the virtual_entries
@@ -568,25 +559,25 @@ add_logger (cherokee_config_node_t *config, cherokee_virtual_server_t *vserver)
 	logger_func_new_t       func_new;
 	cherokee_plugin_info_t *info      = NULL;
 	cherokee_server_t      *srv       = SRV(vserver->server_ref);
-
+	
 	if (cherokee_buffer_is_empty (&config->val)) {
 		PRINT_ERROR_S ("ERROR: A logger must be specified\n");
 		return ret_error;
 	}
 
+	/* Instance a new logger
+	 */
 	ret = cherokee_plugin_loader_get (&srv->loader, config->val.buf, &info);
 	if (ret < ret_ok) {
 		PRINT_MSG ("ERROR: Couldn't load logger module '%s'\n", config->val.buf);
 		return ret_error;
 	}
 
-	/* Instance a new logger
-	 */
 	func_new = (logger_func_new_t) info->instance;
 	if (func_new == NULL)
 		return ret_error;
 
-	ret = func_new ((void **) &vserver->logger, config);
+	ret = func_new ((void **) &vserver->logger, vserver, config);
 	if (ret != ret_ok)
 		return ret;
 
@@ -708,21 +699,20 @@ configure_virtual_server_property (cherokee_config_node_t *conf, void *data)
 		vserver->verify_depth = !!atoi (conf->val.buf);
 
 	} else if (equal_buf_str (&conf->key, "ssl_certificate_file")) {
-		cherokee_buffer_init (&vserver->server_cert);
 		cherokee_buffer_add_buffer (&vserver->server_cert, &conf->val);
 
 	} else if (equal_buf_str (&conf->key, "ssl_certificate_key_file")) {
-		cherokee_buffer_init (&vserver->server_key);
 		cherokee_buffer_add_buffer (&vserver->server_key, &conf->val);
 
 	} else if (equal_buf_str (&conf->key, "ssl_ca_list_file")) {
-		cherokee_buffer_init (&vserver->certs_ca);
 		cherokee_buffer_add_buffer (&vserver->certs_ca, &conf->val);
 
 	} else if (equal_buf_str (&conf->key, "ssl_client_certs")) {
-		cherokee_buffer_init (&vserver->req_client_certs);
 		cherokee_buffer_add_buffer (&vserver->req_client_certs, &conf->val);
 
+	} else if (equal_buf_str (&conf->key, "ssl_ciphers")) {
+		cherokee_buffer_add_buffer (&vserver->ciphers, &conf->val);
+
 	} else {
 		PRINT_MSG ("ERROR: Virtual Server: Unknown key '%s'\n", conf->key.buf);
 		return ret_error;
