@@ -26,7 +26,7 @@ echo "%s"
 
 class Test (TestBase):
     def __init__ (self):
-        TestBase.__init__ (self)
+        TestBase.__init__ (self, __file__)
         self.name = "Rule and: not match"
 
         self.request           = "GET /%s/%s HTTP/1.0\r\n" % (DIR, FILE) 
